@@ -1,4 +1,4 @@
-/* psync.c - Last modified: 29-Mar-2019 (kobayasy)
+/* psync.c - Last modified: 14-Apr-2019 (kobayasy)
  *
  * Copyright (c) 2018-2019 by Yuichi Kobayashi <kobayasy@kobayasy.com>
  *
@@ -47,6 +47,7 @@
 #define BACKUP_DEFAULT   (3*24*60*60)  /* [sec] */
 #define POLL_TIMEOUT 100000  /* [msec] */
 #define LOADBUFFER_SIZE (16*1024)  /* [byte] */
+#define PROGRESS_INTERVAL 1000  /* [msec] */
 
 #ifndef SYMLINK_MAX
 #define SYMLINK_MAX PATH_MAX
@@ -377,6 +378,70 @@ typedef struct {
             dprintf((_priv)->info, __VA_ARGS__); \
     } while (0)
 
+typedef struct {
+    int fd;
+    unsigned long interval;
+    const char *format;
+    intmax_t update, data;
+    struct timespec last;
+} PROGRESS;
+
+#define DIFFTS(_now, _last) \
+    ((long)((_now).tv_sec  - (_last).tv_sec)  *    1000 + \
+           ((_now).tv_nsec - (_last).tv_nsec) / 1000000 )
+
+static int progress_init(PROGRESS *progress, int fd, unsigned long interval, const char *format, intmax_t update) {
+    int status = -1;
+    struct timespec now;
+
+    if (fd != -1) {
+        if (clock_gettime(CLOCK_REALTIME, &now) == -1)
+            goto error;
+        progress->fd = fd;
+        progress->interval = interval;
+        progress->format = format;
+        progress->update = update;
+        progress->data = progress->update;
+        progress->last = now;
+    }
+    status = 0;
+error:
+    return status;
+}
+
+static int progress_update(PROGRESS *progress, intmax_t update) {
+    int status = -1;
+    struct timespec now;
+
+    if (progress->fd != -1) {
+        progress->update = update;
+        if (progress->update != progress->data) {
+            if (clock_gettime(CLOCK_REALTIME, &now) == -1)
+                goto error;
+            if (DIFFTS(now, progress->last) >= progress->interval) {
+                dprintf(progress->fd, progress->format, progress->update);
+                progress->data = progress->update;
+                progress->last = now;
+            }
+        }
+    }
+    status = 0;
+error:
+    return status;
+}
+
+static int progress_term(PROGRESS *progress) {
+    int status = -1;
+
+    if (progress->fd != -1) {
+        if (progress->update != progress->data)
+            dprintf(progress->fd, progress->format, progress->update);
+        progress->fd = -1;
+    }
+    status = 0;
+    return status;
+}
+
 static int lock(PRIV *priv) {
     int status = INT_MIN;
     char loadname[PATH_MAX], *pload;
@@ -814,7 +879,8 @@ error:
 
 static int upload(PRIV *priv) {
     int status = INT_MIN;
-    intmax_t progress;
+    intmax_t uploadsize;
+    PROGRESS progress;
     char loadname[PATH_MAX], *pload;
     unsigned long count;
     FLIST *fsynced;
@@ -823,7 +889,7 @@ static int upload(PRIV *priv) {
     ssize_t n;
     char buffer[LOADBUFFER_SIZE];
 
-    progress = 0;
+    progress_init(&progress, priv->info, PROGRESS_INTERVAL, "U%+jd\n", uploadsize = 0);
     pload = loadname, pload += sprintf(pload, "%s/"SYNCDIR"/"LOCKDIR"/", priv->dirname);
     count = 0;
     for (fsynced = priv->fsynced.next; *fsynced->name; fsynced = fsynced->next) {
@@ -852,8 +918,7 @@ static int upload(PRIV *priv) {
                         goto error;
                     }
                     size -= n;
-                    progress += n;
-                    INFO(priv, "U%+jd\n", progress);
+                    progress_update(&progress, uploadsize += n);
                 }
                 close(fd), fd = -1;
                 break;
@@ -866,8 +931,7 @@ static int upload(PRIV *priv) {
                     status = ERROR_FUPLD;
                     goto error;
                 }
-                progress += size;
-                INFO(priv, "U%+jd\n", progress);
+                progress_update(&progress, uploadsize += size);
                 break;
             }
             if (unlink(loadname) == -1) {
@@ -877,6 +941,7 @@ static int upload(PRIV *priv) {
             break;
         }
     }
+    progress_term(&progress);
     status = 0;
 error:
     if (fd != -1)
@@ -886,7 +951,8 @@ error:
 
 static int download(PRIV *priv) {
     int status = INT_MIN;
-    intmax_t progress;
+    intmax_t downloadsize;
+    PROGRESS progress;
     char loadname[PATH_MAX], *pload;
     unsigned long count;
     FLIST *fsynced;
@@ -896,7 +962,7 @@ static int download(PRIV *priv) {
     char buffer[LOADBUFFER_SIZE];
     struct timeval tv[2];
 
-    progress = 0;
+    progress_init(&progress, priv->info, PROGRESS_INTERVAL, "D%+jd\n", downloadsize = 0);
     pload = loadname, pload += sprintf(pload, "%s/"SYNCDIR"/"LOCKDIR"/", priv->dirname);
     count = 0;
     for (fsynced = priv->fsynced.next; *fsynced->name; fsynced = fsynced->next) {
@@ -925,8 +991,7 @@ static int download(PRIV *priv) {
                         goto error;
                     }
                     size -= n;
-                    progress += n;
-                    INFO(priv, "D%+jd\n", progress);
+                    progress_update(&progress, downloadsize += n);
                 }
                 close(fd), fd = -1;
                 if (chmod(loadname, fsynced->st.mode & (S_IRWXU|S_IRWXG|S_IRWXO)) == -1) {
@@ -948,8 +1013,7 @@ static int download(PRIV *priv) {
                     status = ERROR_FWRITE;
                     goto error;
                 }
-                progress += size;
-                INFO(priv, "D%+jd\n", progress);
+                progress_update(&progress, downloadsize += size);
                 break;
             }
             tv[0].tv_sec = fsynced->st.mtime, tv[0].tv_usec = 0;
@@ -961,6 +1025,7 @@ static int download(PRIV *priv) {
             break;
         }
     }
+    progress_term(&progress);
     status = 0;
 error:
     if (fd != -1)
